using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReflectionIT.Diagnostics {

    internal delegate void InvokeDelegate();

    public partial class NotificationForm : Form {

        private int _listLength = 0;

        public event EventHandler Exit;

        public NotificationForm() {
            InitializeComponent();
        }

        public NotificationForm(int length, string text) : this() {

            notifyIconTrace.Icon = this.Icon;
            notifyIconTrace.Text = text;
            notifyIconTrace.Visible = true;

            this.Text = text;

            _listLength = length;

            this.Location = new Point(-5000, -5000);
            
        }

        protected virtual void OnExit() {
            EventHandler handler = Exit;
            if (handler != null) {
                handler(this, EventArgs.Empty);
            }
        }

        private void OnNotifyIconDoubleClick(object sender, System.EventArgs e) {
            SafeShow();
        }

        private void SafeShow() {
            this.Location = new Point(Screen.PrimaryScreen.WorkingArea.X + Screen.PrimaryScreen.WorkingArea.Width - this.Size.Width,
                          Screen.PrimaryScreen.WorkingArea.Y + Screen.PrimaryScreen.WorkingArea.Height - this.Size.Height);
 
            if (this.InvokeRequired) {
                this.Invoke(new InvokeDelegate(delegate {
                    this.Show();
                }));
            } else {
                this.Show();
            }
        }

        private void listView1_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (listViewTrace.SelectedItems.Count > 0) {
                textBoxMessage.Text = listViewTrace.SelectedItems[0].SubItems[1].Text;
            } else {
                textBoxMessage.Text = String.Empty;
            }
        }

        public void Write(string message, string category) {
            if (listViewTrace.Items.Count > 0) {
                ListViewItem item = listViewTrace.Items[0];
                item.Text = DateTime.Now.ToString();
                item.SubItems[1].Text += message;
                item.Selected = true;
                //textBoxMessage.Text = item.SubItems[1].Text;
            } else
                this.WriteLine(message, category);
        }

        public void Write(string message) {
            this.Write(message, String.Empty);
        }

        public void WriteLine(string message, string category) {
            ListViewItem item = new ListViewItem(new string[] { DateTime.Now.ToString(), message, category });
            listViewTrace.Invoke(new InvokeDelegate(delegate {
                listViewTrace.Items.Add(item);
                item.Selected = true;
                listViewTrace.TopItem = item;
                if (listViewTrace.Items.Count > _listLength) {
                    listViewTrace.Items.RemoveAt(_listLength);
                }
            }));
        }

        private void menuItemTopMost_Click(object sender, System.EventArgs e) {
            menuItemTopMost.Checked = !menuItemTopMost.Checked;
            this.TopMost = menuItemTopMost.Checked;
        }

        private void menuItemExit_Click(object sender, System.EventArgs e) {
            OnExit();
        }

        private void menuItemOpen_Click(object sender, System.EventArgs e) {
            SafeShow();
        }

        public void WriteLine(string message) {
            this.WriteLine(message, String.Empty);
        }

        private void NotificationForm_FormClosing(object sender, FormClosingEventArgs e) {
            e.Cancel = e.CloseReason == CloseReason.UserClosing;
            if (e.Cancel) {
                this.Hide();
            }
        }

        private void NotificationForm_Shown(object sender, EventArgs e) {
            this.Hide();
        }

    }
}